#include <string.h>
#include <jni.h>

#include "tgxf-transmit-pull.h"
#include "tgxf-receive.h"

/*
 * This is where we do the bindings for our TGXF library.
 * Note the naming convention for the functions - this is the
 * way the functions are linked to the correct Java classes.
 */

JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_UploadActivity_tgxfInitSession(JNIEnv* env, jobject thiz, jobject dataToSend, jint qrCodeVersion, jstring filename, jint fps) {
	jboolean isCopy = 0;

	const char * filename_cstr = ((*env)->GetStringUTFChars(env, filename, &isCopy));

	int length = (*env)->GetArrayLength(env, dataToSend);

	return tgxf_init_session((*env)->GetByteArrayElements(env, dataToSend, NULL), length, qrCodeVersion, filename_cstr, fps);
}


JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_UploadActivity_tgxfPullFrame(JNIEnv* env, jobject thiz, jint frameNumber, jbyteArray dataToReceive) {
    QRcode * code;
	tgxf_pull_frame(frameNumber, &code);

	(*env)->SetByteArrayRegion(env, dataToReceive, 0, code->width*code->width, code->data);

	return code->width;
}

JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_UploadActivity_tgxfGetNumFrames(JNIEnv* env, jobject thiz) {
	return tgxf_get_num_frames();
}

// The downloader

JNIEXPORT void JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfReset(JNIEnv* env, jobject thiz) {
	tgxf_reset();
}

//com.midnightcode.thruglassxfer_android.DownloadActivity.decodeTGXFFrame:(Lnet/sourceforge/zbar/Symbol;Ljava/lang/String;)
JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfDecodeTGXFFrame(JNIEnv* env, jobject thiz, jobject symbol, jstring filePath) {
	jboolean isCopy = 0;
	jfieldID Symbol_peer;

	jclass symbolClass = (*env)->GetObjectClass(env, symbol);

	if (symbolClass) {
		Symbol_peer = (*env)->GetFieldID(env, symbolClass, "peer", "J");
		zbar_symbol_t* symbolObj = (zbar_symbol_t*)((*env)->GetLongField(env, symbol, Symbol_peer));

		const char * filePath_cstr = ((*env)->GetStringUTFChars(env, filePath, &isCopy));
		return (int) decode_tgxf_frame(symbolObj, filePath_cstr);
	}
	else {
		return -1;
	}
}

JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfGetCurrentFrameNum(JNIEnv* env, jobject thiz) {
	return tgxf_get_current_frame_num();
}


JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfGetTotalNumFrames(JNIEnv* env, jobject thiz) {
	return tgxf_get_total_num_frames();
}


JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfGetNumErrors(JNIEnv* env, jobject thiz) {
	return tgxf_get_num_errors();
}

JNIEXPORT void JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfSetMaxTimeout(JNIEnv* env, jobject thiz, jint num) {
	tgxf_set_max_timeout(num);
}

JNIEXPORT void JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfSetMaxErrors(JNIEnv* env, jobject thiz, jint num) {
	tgxf_set_max_errors(num);
}

JNIEXPORT jint JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfGetFPS(JNIEnv* env, jobject thiz) {
	return tgxf_get_fps();
}

JNIEXPORT jstring JNICALL Java_com_midnightcode_thruglassxfer_1android_DownloadActivity_tgxfGetFilename(JNIEnv* env, jobject thiz) {
	return (*env)->NewStringUTF(env, tgxf_get_filename());
}
