package com.midnightcode.thruglassxfer_android;

import java.util.List;

import net.sourceforge.zbar.Config;
import net.sourceforge.zbar.Image;
import net.sourceforge.zbar.ImageScanner;
import net.sourceforge.zbar.Symbol;
import net.sourceforge.zbar.SymbolSet;
import android.R.string;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PreviewCallback;
import android.hardware.Camera.Size;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
/* Import ZBar Class files */

public class DownloadActivity extends ActionBarActivity {
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_download);

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        
		if (savedInstanceState == null) {
			getSupportFragmentManager().beginTransaction()
					.add(R.id.container, new PlaceholderFragment()).commit();
		}
		
		tgxfReset();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		// Inflate the menu; this adds items to the action bar if it is present.
		//getMenuInflater().inflate(R.menu.download, menu);
		return false;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_settings) {
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * A placeholder fragment containing a simple view.
	 */
	public static class PlaceholderFragment extends Fragment {

	    private static boolean isUsingContinuousFocus = false;
		private Camera mCamera;
	    private CameraPreview mPreview;
	    private Handler autoFocusHandler;
	    private int lastTGXFState;
	    
	    ImageScanner scanner;
		
		public PlaceholderFragment() {
		}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container,
				Bundle savedInstanceState) {
			View rootView = inflater.inflate(R.layout.fragment_download,
					container, false);
	        
	        autoFocusHandler = new Handler();
	        mCamera = getCameraInstance();
	        lastTGXFState = TGXF_RECEIVE_STATE_WAITING_FOR_FIRST_FRAME;
	        
	        /* Instance barcode scanner */
	        scanner = new ImageScanner();
	        scanner.setConfig(0, Config.X_DENSITY, 3);
	        scanner.setConfig(0, Config.Y_DENSITY, 3);

	        mPreview = new CameraPreview(rootView.getContext(), mCamera, previewCb, autoFocusCB);
	        FrameLayout preview = (FrameLayout)rootView.findViewById(R.id.cameraPreview);
	        preview.addView(mPreview);
	        
			return rootView;
		}
		
		@Override
		public void onDestroyView() {
			super.onDestroyView();
			releaseCamera();
		}


		
    /** A safe way to get an instance of the Camera object. */
    public static Camera getCameraInstance(){
        Camera c = null;
        try {
            c = Camera.open();
            Parameters p = c.getParameters();
            p.setFlashMode(Parameters.FLASH_MODE_OFF);
            
            List<String> focusModes = p.getSupportedFocusModes();

		    String CAF_PICTURE = Parameters.FOCUS_MODE_CONTINUOUS_PICTURE, 
		           CAF_VIDEO = Parameters.FOCUS_MODE_CONTINUOUS_VIDEO, 
		           supportedMode = focusModes
		                   .contains(CAF_PICTURE) ? CAF_PICTURE : focusModes
		                   .contains(CAF_VIDEO) ? CAF_VIDEO : "";

		    if (!supportedMode.equals("")) {
		        p.setFocusMode(supportedMode);
		        isUsingContinuousFocus  = true;
		    }
            
            c.setParameters(p);
        } catch (Exception e){
        	Log.v("Download", "CAMERA START FAILED!");
        }
        return c;
    }

    private void releaseCamera() {
        if (mCamera != null) {
        	mCamera.stopPreview();
            mCamera.setPreviewCallback(null);
            mCamera.release();
            mCamera = null;
        }
    }

    private Runnable doAutoFocus = new Runnable() {
            public void run() {
            	if (isUsingContinuousFocus || mCamera == null) {
            		return;
            	}
                    mCamera.autoFocus(autoFocusCB);
            }
        };

    public static final int TGXF_RECEIVE_STATE_IN_PROGRESS = 0;
    public static final int TGXF_RECEIVE_STATE_FAILED = 1;
    public static final int TGXF_RECEIVE_STATE_SUCCEEDED = 2;
    public static final int TGXF_RECEIVE_STATE_INTERNAL_ERROR = 3;  
    public static final int TGXF_RECEIVE_STATE_WAITING_FOR_FIRST_FRAME = -10; // Not in library but added for UI state
      
	private boolean isFinished() {
		switch (lastTGXFState) {
			case TGXF_RECEIVE_STATE_INTERNAL_ERROR:
			case TGXF_RECEIVE_STATE_SUCCEEDED:
			case TGXF_RECEIVE_STATE_FAILED:
				return true;
				
			case TGXF_RECEIVE_STATE_WAITING_FOR_FIRST_FRAME:
			case TGXF_RECEIVE_STATE_IN_PROGRESS:
				return false;
		}
		return false;
	}
    
    PreviewCallback previewCb = new PreviewCallback() {
            public void onPreviewFrame(byte[] data, Camera camera) {
                Camera.Parameters parameters = camera.getParameters();
                Size size = parameters.getPreviewSize();

                if (isFinished() ) {
                	return;
                }
                
                Image barcode = new Image(size.width, size.height, "Y800");
                barcode.setData(data);

                int result = scanner.scanImage(barcode);
                
                if (result != 0) {                    
                    SymbolSet syms = scanner.getResults();
                    for (Symbol sym : syms) {
                        DownloadActivity activity = ((DownloadActivity) getActivity());
                    	
                        TextView frameLabel = (TextView) activity.findViewById(R.id.downloadFrameLabel);
                        TextView progressLabel = (TextView) activity.findViewById(R.id.downloadProgressLabel);
                        ProgressBar progressBar = (ProgressBar) activity.findViewById(R.id.downloadProgressBar);
                        TextView errorsLabel = (TextView) activity.findViewById(R.id.downloadErrorsLabel);	

                        String path = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).toString() +"/"+ activity.tgxfGetFilename();
                    	
                        lastTGXFState = activity.tgxfDecodeTGXFFrame(sym, path);

                        if (lastTGXFState == TGXF_RECEIVE_STATE_INTERNAL_ERROR) {
                            progressLabel.setText("Internal error");
                            return;
                       }
                    	
                        if (lastTGXFState == TGXF_RECEIVE_STATE_FAILED) {
                            progressLabel.setText("Transfer failed");
                            return;
                       }
                       
                       if (lastTGXFState != TGXF_RECEIVE_STATE_IN_PROGRESS) {

                    	   Toast.makeText(getActivity().getBaseContext(), getString(R.string.download_completed), Toast.LENGTH_LONG).show();
                    	   progressLabel.setText("Completed");
                           return;
                       }
                       
                       int frameNum = activity.tgxfGetCurrentFrameNum();
                       int total = activity.tgxfGetTotalNumFrames();
                       int errors = activity.tgxfGetNumErrors();
                       
                       SharedPreferences settings = activity.getSharedPreferences("tgxf", 0);
                       int maxErrors = settings.getInt("MaxErrors", 0);
                       int fps = activity.tgxfGetFPS();
                       
                       if (fps <= 0) {
                    	   progressLabel.setText("Getting framerate");
                    	   return;
                       }
                       
                       if (total <= 0) {
                    	   progressLabel.setText("Getting number of frames");
                    	   return;
                       }
                       
                       activity.tgxfSetMaxErrors(maxErrors);
                       activity.tgxfSetMaxTimeout(settings.getInt("Timeout", 1000));
                       
                       int seconds = (total-frameNum)/fps;
                       
                       errorsLabel.setText("Errors " +errors+" of "+maxErrors+" (Max)");
                       frameLabel.setText("Frame: "+ /*Math.max(Math.min(frameNum, total), 0)*/ frameNum +" of "+ total);
                       progressLabel.setText(""+seconds/60+"min "+Math.max(0, seconds%60)+"sec to go");
                       progressBar.setProgress(frameNum*progressBar.getMax()/total);
                    
                    }
                }
            }            
        };

    // Mimic continuous auto-focusing
    AutoFocusCallback autoFocusCB = new AutoFocusCallback() {
            public void onAutoFocus(boolean success, Camera camera) {
                autoFocusHandler.postDelayed(doAutoFocus, 2000);
            }
        };
	}
	
    static {
        System.loadLibrary("iconv");
        System.loadLibrary("ThruGlassXfer");
    } 
    
    native void tgxfReset();
    native int tgxfDecodeTGXFFrame(Symbol sym, String filePath);
    native int tgxfGetCurrentFrameNum();
    native int tgxfGetTotalNumFrames();
    native int tgxfGetNumErrors();
    native void tgxfSetMaxErrors(int num);
    native void tgxfSetMaxTimeout(int num);
    native int tgxfGetFPS();
    native string tgxfGetFilename();

}
