package com.midnightcode.thruglassxfer_android;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.app.ActionBarActivity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import ar.com.daidalos.afiledialog.FileChooserActivity;

public class UploadActivity extends ActionBarActivity {

	private static final int MaxFileSizeBytes = 65535;
	private int frameNumber = 0;
	private Timer showFrameTimer;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_upload);

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        
		if (savedInstanceState == null) {
			getSupportFragmentManager().beginTransaction()
					.add(R.id.container, new PlaceholderFragment()).commit();
		}
	}

	@Override
	protected void onStop() {
		super.onStop();
		
	    if (showFrameTimer != null) {
	    	showFrameTimer.cancel();
	    	showFrameTimer = null;
	    }
		
		if (frameNumber > 0) {
			stopTGXFSession();
			frameNumber = 0;
		}
	}
	
    public native void tgxfInitSession(byte[] dataToSend, int qrCodeVersion, String string, int fps);
    public native int tgxfPullFrame(int frameNumber, byte[] dataToReceive);
    public native int tgxfGetNumFrames();
    
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		// Inflate the menu; this adds items to the action bar if it is present.
		//getMenuInflater().inflate(R.menu.upload, menu);
		return false;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_settings) {
			return true;
		}
		return super.onOptionsItemSelected(item);
	}
	
	public void onChooseFileButtonClick(View view) {
		Intent intent = new Intent(this, FileChooserActivity.class);
	     this.startActivityForResult(intent, 0);
	     
	     if (showFrameTimer != null) {
			showFrameTimer.cancel();
			showFrameTimer = null;
	     }
	}
	
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode == Activity.RESULT_OK) {
            String filePath = "";
            
            Bundle bundle = data.getExtras();
            if(bundle != null)
            {
                    if(bundle.containsKey(FileChooserActivity.OUTPUT_NEW_FILE_NAME)) {
                            File folder = (File) bundle.get(FileChooserActivity.OUTPUT_FILE_OBJECT);
                            String name = bundle.getString(FileChooserActivity.OUTPUT_NEW_FILE_NAME);
                            filePath = folder.getAbsolutePath() + "/" + name;
                    } else {
                            File file = (File) bundle.get(FileChooserActivity.OUTPUT_FILE_OBJECT);
                            filePath = file.getAbsolutePath();
                    }
            }
                    
                    File file = new File(filePath);
                    int size = (int) file.length();
                    
                    if (size > MaxFileSizeBytes) {
                    	new AlertDialog.Builder(this)
                        .setTitle("File too large")
                        .setMessage("Your chosen file is too large. Please choose one which is under " + MaxFileSizeBytes + " bytes.")
                        .setIcon(android.R.drawable.ic_dialog_alert)
                         .show();
                    }
                    else {
                    byte[] bytes = new byte[size];
                    try {
                        BufferedInputStream buf = new BufferedInputStream(new FileInputStream(file));
                        buf.read(bytes, 0, bytes.length);
                        buf.close();
                    } catch (FileNotFoundException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    } catch (IOException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                                 
                    startTGXFSession(bytes, filePath);
                    Button startPauseToggledButton = (Button) findViewById(R.id.startPauseToggledButton);
                    startPauseToggledButton.setVisibility(View.VISIBLE);
                    startPauseToggledButton.setEnabled(true);
                    startPauseToggledButton.setText("Begin");
                    frameNumber = 1;
                    }
        }
    }
	
	public void onStartPauseToggled(View view) {
        Button startPauseToggledButton = (Button) findViewById(R.id.startPauseToggledButton);
        
		if (showFrameTimer != null) {
			showFrameTimer.cancel();
			showFrameTimer = null;

            startPauseToggledButton.setText("Resume");
		}
		else {	        
            startPauseToggledButton.setText("Pause");
            showFrameTimer = new Timer();
			showFrameTimer.scheduleAtFixedRate(new TimerTask() {
				
				@Override
				public void run() {
					pullNextTGXFFrame();
				}
			}, 0, 1000/getFPS());
		}
	}
    
	private int getFPS() {
        SharedPreferences settings = getSharedPreferences("tgxf", 0);
        return settings.getInt("FramesPerSecond", 5);
	}

	private int getQRCodeVersion() {
        SharedPreferences settings = getSharedPreferences("tgxf", 0);
        return settings.getInt("FrameResolution", 1);
	}
	
	private void startTGXFSession(byte[] bytes, String filePath) {
		int version = getQRCodeVersion();
		tgxfInitSession(bytes, version, filePath, getFPS());
		
	    frameNumber = 1;
	    pullNextTGXFFrame();
	}

	private void pullNextTGXFFrame() {
	    final int totalFrames = tgxfGetNumFrames();
	    
	    if (frameNumber <= totalFrames+5) { // plus control frames
	    	byte []dataToReceive = new byte[128000];
	    	int width = tgxfPullFrame(frameNumber, dataToReceive);
	    	
	    	Bitmap bmp = Bitmap.createBitmap(width, 
	    		    width, Bitmap.Config.ARGB_8888);
	    	
            for (int i=0; i< width * width; i++) {
            	int pixel =  ((dataToReceive[i] & 0x1) == 0) ? 0xffffffff : 0xff000000;
                bmp.setPixel(i%width, i/width, pixel);
            }
            
            final BitmapDrawable draw = new BitmapDrawable(getResources(), bmp);
            draw.setAntiAlias(false);
            draw.setFilterBitmap(false);
            draw.setDither(false);
            
            this.runOnUiThread(new Runnable(){
                public void run(){
            ImageView barcodeDisplayImage = (ImageView) findViewById(R.id.barcodeDisplayImage);
            barcodeDisplayImage.setImageDrawable(draw);
            
            int frameNum = Math.max(Math.min(frameNumber, totalFrames), 0);
            
            TextView frameLabel = (TextView) findViewById(R.id.frameLabel);
            TextView progressLabel = (TextView) findViewById(R.id.progressLabel);
            ProgressBar progressBar = (ProgressBar) findViewById(R.id.downloadProgressBar);
            
            int seconds = (int) ((totalFrames-frameNum)/(float)getFPS());
            frameLabel.setText("Frame: "+frameNum+" of " + totalFrames);
            progressLabel.setText("" + seconds/60 + "min "+ Math.max(0, seconds%60) +"sec to go");
            progressBar.setProgress(frameNumber*progressBar.getMax()/totalFrames);
            }});
            
            frameNumber++;
	    }
	    else {
	    	stopTGXFSession();
	    }
	}

	private void stopTGXFSession() {
		if (showFrameTimer != null) {
			showFrameTimer.cancel();
			showFrameTimer = null;
		}
	}

	/**
	 * A placeholder fragment containing a simple view.
	 */
	public static class PlaceholderFragment extends Fragment {

		public PlaceholderFragment() {
		}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container,
				Bundle savedInstanceState) {
			View rootView = inflater.inflate(R.layout.fragment_upload,
					container, false);
			return rootView;
		}
	}

	static {
		System.loadLibrary("iconv");
		System.loadLibrary("ThruGlassXfer");
	}
}
